@@ -15,6 +15,7 @@ from sklearn.model_selection import train_test_split
 from sklearn.ensemble import GradientBoostingRegressor
 from sklearn.model_selection import train_test_split
 from sklearn.preprocessing import MinMaxScaler
+from sklearn.linear_model import LinearRegression
 import holidays
 import matplotlib.pyplot as plt
 import holidays
@@ -54,7 +55,7 @@ def get_temp_smoothed_fr(start_date: str, end_date: str) -> pd.DataFrame:
     Retour :
     -------
     pd.DataFrame
-        DataFrame avec les colonnes ['datetime', 'temperature'] représentant la température moyenne lissée.
+        DataFrame avec les colonnes ['timestamp', 'temperature'] représentant la température moyenne lissée.
     """
 
     # 1. Définition des grandes villes françaises pour lisser les données à l'échelle nationale
@@ -90,24 +91,24 @@ def get_temp_smoothed_fr(start_date: str, end_date: str) -> pd.DataFrame:
 
             # 5. Construction du DataFrame pour la ville courante
             df = pd.DataFrame({
-                'datetime': data['hourly']['time'],
+                'timestamp': data['hourly']['time'],
                 city: data['hourly']['temperature_2m']
             })
-            df['datetime'] = pd.to_datetime(df['datetime'])  # Conversion en datetime
-            df.set_index('datetime', inplace=True)  # Mise en index sur la date/heure
+            df['timestamp'] = pd.to_datetime(df['timestamp'])  # Conversion en datetime
+            df.set_index('timestamp', inplace=True)  # Mise en index sur la date/heure
 
             city_dfs.append(df)  # 6. Ajout du DataFrame ville à la liste
 
         except Exception as e:
             print(f"Error with {city}: {e}")  # 7. Gestion simple des erreurs
 
-    # 8. Fusion de tous les DataFrames de villes selon l'index datetime (concaténation horizontale)
+    # 8. Fusion de tous les DataFrames de villes selon l'index timestamp (concaténation horizontale)
     df_all = pd.concat(city_dfs, axis=1)
 
     # 9. Calcul de la moyenne horaire sur toutes les villes (lissage national)
     df_all['temperature'] = df_all.mean(axis=1)
 
-    # 10. Retourne uniquement la colonne datetime et la température moyenne lissée (réinitialisation de l'index)
+    # 10. Retourne uniquement la colonne timestamp et la température moyenne lissée (réinitialisation de l'index)
     return df_all[['temperature']].reset_index()
 
 def eval_forecast(df, temp_df, cal_year=None, datetime_col='timestamp', target_col='MW', plot_chart=False, save_path=None):
@@ -119,7 +120,7 @@ def eval_forecast(df, temp_df, cal_year=None, datetime_col='timestamp', target_c
     df : pd.DataFrame
         Données historiques contenant au moins les colonnes datetime_col et target_col.
     temp_df : pd.DataFrame
-        Données de température avec colonnes ['datetime', 'temperature'].
+        Données de température avec colonnes ['timestamp', 'temperature'].
     cal_year : int ou None, optionnel (par défaut None)
         Année civile pour laquelle générer la prévision. Obligatoire si plot_chart=True.
     datetime_col : str, optionnel (par défaut 'timestamp')
@@ -176,8 +177,8 @@ def eval_forecast(df, temp_df, cal_year=None, datetime_col='timestamp', target_c
     # 9. Préparation et nettoyage des données de température
     temp_df = temp_df.copy()
     # Conversion en datetime avec fuseau UTC, puis suppression du fuseau pour homogénéité
-    temp_df[datetime_col] = pd.to_datetime(temp_df['datetime'], utc=True).dt.tz_localize(None)
-    temp_df = temp_df.drop(columns=['datetime'])
+    temp_df[datetime_col] = pd.to_datetime(temp_df['timestamp'], utc=True).dt.tz_localize(None)
+    #temp_df = temp_df.drop(columns=['timestamp'])
 
     # 10. Fusion des données météo avec les données principales sur datetime_col
     df = pd.merge(df, temp_df, on=datetime_col, how='left')
@@ -283,7 +284,7 @@ def eval_forecast(df, temp_df, cal_year=None, datetime_col='timestamp', target_c
         else:
             fig.show()
 
-    # 19. Retourne les résultats : datetime, valeur réelle, et prévision
+    # 19. Retourne les résultats : timestamp, valeur réelle, et prévision
     return test_df[[datetime_col, target_col, 'forecast']]
 
 def get_spot_price_fr(start_date: str, end_date: str) -> pd.DataFrame:
@@ -733,4 +734,139 @@ def calculate_prem_risk_shape(forecast_df: pd.DataFrame,pfc_df: pd.DataFrame,spo
 
 
 
+def calculate_prem_risk_thermo(load_curve_df: pd.DataFrame, temp_df: pd.DataFrame, spot_df: pd.DataFrame, quantile: int = 70, n_scenarios: int = 1000, temp_variation_range: float = 5.0, plot_chart: bool = False, save_path: Optional[str] = None) -> float:
+    """
+    Calcule la prime de risque thermique en simulant des scénarios de température 
+    s'écartant progressivement de la température normale.
 
+    Paramètres :
+    -----------
+    load_curve_df : pd.DataFrame
+        Données de consommation avec colonnes ['timestamp', 'MW'].
+    temp_df : pd.DataFrame
+        Données de température avec colonnes ['timestamp', 'temperature'].
+    spot_df : pd.DataFrame
+        Données de prix spot avec colonnes ['delivery_from', 'price_eur_per_mwh'].
+    quantile : int, par défaut 70
+        Quantile (en %) à extraire de la distribution des primes calculées.
+    n_scenarios : int, par défaut 1000
+        Nombre de scénarios de température simulés.
+    temp_variation_range : float, par défaut 5.0
+        Plage d'écart simulé autour de la température normale (°C).
+    plot_chart : bool, par défaut False
+        Si True, affiche un graphique interactif (Plotly) des primes simulées triées.
+    save_path : str, optionnel
+        Si défini, sauvegarde le graphique interactif au format HTML à ce chemin.
+
+    Retour :
+    --------
+    float
+        Valeur du quantile demandé (en €/MWh), mesurant la prime de risque thermique.
+    """
+
+    # 1. Définition des mois d'hiver et d'été
+    winter_months = [11, 12, 1, 2, 3]
+    summer_months = [6, 7, 8, 9]
+
+    # 2. Conversion des colonnes timestamps en datetime UTC
+    load_curve_df['timestamp'] = pd.to_datetime(load_curve_df['timestamp'], utc=True)
+    temp_df['timestamp'] = pd.to_datetime(temp_df['timestamp'], utc=True)
+    spot_df['delivery_from'] = pd.to_datetime(spot_df['delivery_from'], utc=True)
+
+    # 3. Fusion "asof" des données consommation avec température sur timestamp le plus proche
+    merged = pd.merge_asof(
+        load_curve_df.sort_values('timestamp'),
+        temp_df.sort_values('timestamp'),
+        left_on='timestamp',
+        right_on='timestamp',
+        direction='nearest'
+    )
+
+    # 4. Extraction du mois pour différencier saison hiver/été
+    merged['month'] = merged['timestamp'].dt.month
+
+    # 5. Calcul du gradient consommation-température pour l'hiver par régression linéaire
+    winter_df = merged[merged['month'].isin(winter_months)]
+    X_winter = winter_df['temperature'].values.reshape(-1, 1)
+    y_winter = winter_df['MW'].values
+    model_winter = LinearRegression().fit(X_winter, y_winter)
+    gradient_winter = model_winter.coef_[0]
+
+    # 6. Calcul du gradient consommation-température pour l'été
+    summer_df = merged[merged['month'].isin(summer_months)]
+    X_summer = summer_df['temperature'].values.reshape(-1, 1)
+    y_summer = summer_df['MW'].values
+    model_summer = LinearRegression().fit(X_summer, y_summer)
+    gradient_summer = model_summer.coef_[0]
+
+    # 7. Filtrage des prix spot sur la période étudiée
+    spot_period = spot_df[
+        (spot_df['delivery_from'] >= load_curve_df['timestamp'].min()) &
+        (spot_df['delivery_from'] <= load_curve_df['timestamp'].max())
+    ]
+    avg_spot_price = spot_period['price_eur_per_mwh'].mean()
+
+    # 8. Extraction de la température normale sur la période
+    temp_period = temp_df[
+        (temp_df['timestamp'] >= load_curve_df['timestamp'].min()) &
+        (temp_df['timestamp'] <= load_curve_df['timestamp'].max())
+    ].copy()
+
+    temp_normal = temp_period['temperature'].values
+    month_nums = temp_period['timestamp'].dt.month.values
+
+    # 9. Attribution des gradients selon la saison pour chaque timestamp
+    gradients = np.where(
+        np.isin(month_nums, winter_months),
+        gradient_winter,
+        np.where(np.isin(month_nums, summer_months), gradient_summer, 0)
+    )
+
+    # 10. Simulation de n_scenarios de températures variant linéairement autour de la température normale
+    temp_scenarios = np.array([
+        temp_normal + np.linspace(-temp_variation_range, temp_variation_range, n_scenarios)[i]
+        for i in range(n_scenarios)
+    ])
+
+    # 11. Calcul de l'écart en MW pour chaque scénario
+    delta_MW = (temp_scenarios - temp_normal) * gradients
+
+    # 12. Calcul des primes en €/MWh pour chaque scénario
+    premiums = np.abs(delta_MW * avg_spot_price).mean(axis=1)
+
+    # 13. Extraction du quantile demandé (conversion % -> fraction)
+    quantile_norm = quantile / 100
+    quantile_value = np.quantile(premiums, quantile_norm)
+
+    # 14. Affichage graphique (optionnel)
+    if plot_chart or save_path:
+        sorted_premiums = np.sort(premiums)
+        fig = go.Figure()
+        fig.add_trace(go.Scatter(
+            y=sorted_premiums,
+            x=list(range(1, len(sorted_premiums) + 1)),
+            mode='lines+markers',
+            name='Prime risque thermique',
+            line=dict(color='orange')
+        ))
+        fig.add_hline(y=quantile_value, line_dash="dash", line_color="red",
+                      annotation_text=f"{quantile}e percentile = {quantile_value:.2f} €/MWh",
+                      annotation_position="top right")
+        fig.update_layout(
+            title="Distribution des primes de risque thermique",
+            xaxis_title="Index (trié)",
+            yaxis_title="Prime thermique (€/MWh)",
+            plot_bgcolor='black',
+            paper_bgcolor='black',
+            font=dict(color='white'),
+            hovermode='closest'
+        )
+        if save_path:
+            fig.write_html(save_path)
+            print(f"Graphique interactif enregistré : {save_path}")
+        if plot_chart:
+            fig.show()
+
+    # 15. Affichage console et retour de la valeur du quantile
+    print(f"Quantile {quantile}% prime risque thermique = {quantile_value:.4f} €/MWh")
+    return quantile_value
